@@ -1,21 +1,16 @@
-#: Makefile/GraphViz.pm
-#: Plot the detailed structure of Makefiles
-#:   using GraphViz
-#: v0.10
-#: Copyright (c) 2005 Agent Zhang
-#: 2005-09-30 2005-10-31
-
 package Makefile::GraphViz;
 
+use 5.006001;
 use strict;
 use warnings;
+use vars qw { $VERSION };
 
 use GraphViz;
 use base 'Makefile::Parser';
 
-$Makefile::Parser::Strict = 0;
+$VERSION = '0.11';
 
-our $VERSION = '0.10';
+$Makefile::Parser::Strict = 0;
 
 our $IDCounter = 0;
 
@@ -52,7 +47,33 @@ my %InitArgs = (
 
 our %Nodes;
 
-sub plot {
+sub _gen_id () {
+    return ++$IDCounter;
+}
+
+sub _trim_path ($) {
+    my $s = shift;
+    $s =~ s/.+(.{5}[\\\/].*)$/...$1/o;
+    $s =~ s/\\/\\\\/g;
+    return $s;
+}
+
+sub _trim_cmd ($) {
+    my $s = shift;
+    $s =~ s/((?:\S+\s+){2})\S.*/$1.../o;
+    $s =~ s/\\/\\\\/g;
+    return $s;
+}
+
+sub _find ($@) {
+    my $elem = shift;
+    foreach (@_) {
+        return 1 if $elem eq $_;
+    }
+    return undef;
+}
+
+sub plot ($$@) {
     my $self = shift;
     my $root_name = shift;
     my %opts = @_;
@@ -115,11 +136,11 @@ sub plot {
         %Nodes = ();
     }
 
-    return $gv if find($root_name, @exclude);
+    return $gv if _find($root_name, @exclude);
     #warn $gv;
     my $is_virtual = 0;
     if (!$Nodes{$root_name}) {
-        my $short_name = trim_path($root_name);
+        my $short_name = _trim_path($root_name);
         #$short_name =~ s/\\/\//g;
         #warn $short_name, "\n";
         if ($normal_nodes{$root_name}) {
@@ -137,13 +158,13 @@ sub plot {
         return $gv;
     }
     #warn "GraphViz: $gv\n";
-    return $gv if !$root or find($root_name, @end_with);
+    return $gv if !$root or _find($root_name, @end_with);
 
     my $lower_node;
     my @cmds = $root->commands;
     if (!$trim_mode and @cmds) {
-        $lower_node = gen_id();
-        my $cmds = join("\n", map { trim_cmd($_); } @cmds);
+        $lower_node = _gen_id();
+        my $cmds = join("\n", map { _trim_cmd($_); } @cmds);
         $gv->add_node($lower_node, label => $cmds, %cmd_style);
         $gv->add_edge(
             $lower_node => $root_name,
@@ -156,7 +177,7 @@ sub plot {
     my @depends = $root->depends;
     foreach (@depends) {
         #warn "$_\n";
-        next if find($_, @exclude);
+        next if _find($_, @exclude);
         $gv->add_edge(
             $_ => $lower_node,
             $is_virtual ? (style => 'dashed') : ());
@@ -167,30 +188,14 @@ sub plot {
     return $gv;
 }
 
-sub gen_id {
-    return ++$IDCounter;
-}
-
-sub trim_path {
-    my $s = shift;
-    $s =~ s/.+(.{5}[\\\/].*)$/...$1/o;
-    $s =~ s/\\/\\\\/g;
-    return $s;
-}
-
-sub trim_cmd {
-    my $s = shift;
-    $s =~ s/((?:\S+\s+){2})\S.*/$1.../o;
-    $s =~ s/\\/\\\\/g;
-    return $s;
-}
-
-sub find {
-    my $elem = shift;
-    foreach (@_) {
-        return 1 if $elem eq $_;
+sub plot_all ($) {
+    my $self = shift;
+    my $gv = GraphViz->new(%InitArgs);
+    %Nodes = ();
+    for my $target ($self->targets) {
+        $self->plot($target, gv => $gv);
     }
-    return undef;
+    $gv;
 }
 
 1;
@@ -198,7 +203,11 @@ __END__
 
 =head1 NAME
 
-Makefile::GraphViz - Plot the Detailed Structure of Makefiles Using GraphViz                        
+Makefile::GraphViz - Plot the Detailed Structure of Makefiles Using GraphViz
+
+=head1 VERSION
+
+This document describes Makefile::GraphViz 0.11 released on 3 March, 2007.
 
 =head1 SYNOPSIS
 
@@ -207,19 +216,20 @@ Makefile::GraphViz - Plot the Detailed Structure of Makefiles Using GraphViz
   $parser = Makefile::GraphViz->new;
   $parser->parse('Makefile');
 
-  # plot the tree rooted at the install target in Makefile:
+  # plot the tree rooted at the 'install' goal in Makefile:
   $gv = $parser->plot('install');  # A GraphViz object returned.
   $gv->as_png('install.png');
 
-  # plot the tree rooted at the default target in Makefile:
+  # plot the tree rooted at the 'default' goal in Makefile:
   $gv = $parser->plot;
   $gv->as_png('default.png');
 
-  # plot the forest consists of all the targets in Makefile:
+  # plot the forest consists of all the goals in Makefile:
   $gv = $parser->plot_all;
   $gv->as_png('default.png');
 
-  # you can also invoke all the methods inherited from the Makefile::Parser class:
+  # you can also invoke all the methods
+  # inherited from the Makefile::Parser class:
   @targets = $parser->targets;
 
 =head1 DESCRIPTION
@@ -230,10 +240,14 @@ CPAN module named L<GraphViz::Makefile> which did the same thing. However, the
 pictures generated by L<GraphViz::Makefile> is oversimplified in my opinion, so
 a much complex one is still needed.
 
-B<IMPORTANT!>
-This stuff is highly experimental and is currently at B<ALPHA> stage, so
-production use is strongly discouraged. Anyway, I have the plan to 
-improve this stuff unfailingly.
+B<WARNING> This module is highly experimental and is currently at
+B<PRE-ALPHA> stage, so production use is strongly discouraged right now.
+Anyway, I have the plan to improve this stuff unfailingly.
+
+=head1 SAMPLE PICTURES
+
+Browse L<http://search.cpan.org/src/AGENT/Makefile-GraphViz-0.11/samples.html>
+for some sample output graphs.
 
 =head1 INSTALLATION
 
@@ -254,12 +268,12 @@ Additionally this class also provides some more methods on its own right.
 
 =over
 
-=item plot($target, ...)
+=item $graphviz = plot($target, ...)
 
 This method is essential to the class. Users invoke this method to plot the specified
 Makefile target. If the argument is absent, the default target in the Makefile will
 be used. It will return a L<GraphViz> object, on which you can later call the
--E<gt>as_png or -E<gt>as_text method to obtain the final graphical output.
+C<as_png> or C<as_text> method to obtain the final graphical output.
 
 The argument can both be the target's name and a Makefile::Target object. If the
 given target can't be found in Makefile, the target will be plotted separately.
@@ -380,7 +394,9 @@ nodes will be plotted.
 
 The entries in this option's list are forced to be the
 virtual nodes. Virtual nodes are those Makefile targets
-with no real files corresponding to them.
+with no real files corresponding to them, which are generally
+called "phony targets" in the GNU make Manual and "pseudo targets"
+in MS NMAKE's docs.
 
 =item vir_node_style
 
@@ -400,36 +416,35 @@ border.
 
 =back
 
+=item $graphviz = plot_all()
+
+Plot all the (root) goals appeared in the Makefile.
+
 =back
 
 =head2 EXPORT
 
 None by default.
 
-=head1 EXAMPLES
-
-Browse L<http://search.cpan.org/src/AGENT/Makefile-GraphViz-0.10/samples.html>
-for some sample output graphs.
-
 =head2 INTERNAL FUNCTIONS
 
 Internal functions should not be used directly.
 
 =over
 
-=item gen_id
+=item _gen_id
 
 Generate a unique id for command node.
 
-=item trim_path
+=item _trim_path
 
 Trim the path to a more readable form.
 
-=item trim_cmd
+=item _trim_cmd
 
 Trim the shell command to a more friendly size.
 
-=item find
+=item _find
 
 If the given element is found in the given list, this
 function will return 1; otherwise, a false value is
@@ -437,23 +452,47 @@ returned.
 
 =back
 
+=head1 TODO
+
+=over
+
+=item *
+
+Add support for the various options provided by the
+C<plot> method to the C<plot_all> method.
+
+=item *
+
+Use L<Params::Util> to check the validity of the
+method arguments.
+
+=item *
+
+Use the next generation of L<Makefile::Parser> to do
+the underlying parsing job.
+
+=back
+
 =head1 CODE COVERAGE
 
-I use L<Devel::Cover> to test the code coverage of my tests, below is the 
-L<Devel::Cover> report on this module test suite.
+I use L<Devel::Cover> to test the code coverage of my tests,
+below is the L<Devel::Cover> report on this module test suite.
 
-    ---------------------------- ------ ------ ------ ------ ------ ------ ------
-    File                           stmt   bran   cond    sub    pod   time  total
-    ---------------------------- ------ ------ ------ ------ ------ ------ ------
-    .../lib/Makefile/GraphViz.pm  100.0   90.9   69.0  100.0  100.0  100.0   90.9
-    Total                         100.0   90.9   69.0  100.0  100.0  100.0   90.9
-    ---------------------------- ------ ------ ------ ------ ------ ------ ------
+  ---------------------------- ------ ------ ------ ------ ------ ------ ------
+  File                           stmt   bran   cond    sub    pod   time  total
+  ---------------------------- ------ ------ ------ ------ ------ ------ ------
+  .../lib/Makefile/GraphViz.pm  100.0   93.2   71.4  100.0  100.0   61.5   92.1
+  ---------------------------- ------ ------ ------ ------ ------ ------ ------
 
-=head1 REPOSITORY
+=head1 SOURCE CONTROL
 
 For the very latest version of this module, check out the source from
-L<https://svn.berlios.de/svnroot/repos/makefilegv> (Subversion). There is
-anonymous access to all.
+the SVN repos below:
+
+L<https://svn.berlios.de/svnroot/repos/makefilegv>
+
+There is anonymous access to all. If you'd like a commit bit, please let
+me know. :)
 
 =head1 BUGS
 
@@ -466,13 +505,14 @@ L<gvmake>, L<GraphViz>, L<Makefile::Parser>.
 
 =head1 AUTHOR
 
-Agent Zhang, E<lt>agent2002@126.comE<gt>
+Agent Zhang E<lt>agentzh@gmail.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2005 Agent Zhang.
+Copyright (C) 2005, 2006, 2007 by Agent Zhang. All rights reserved.
 
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself.
+This library is free software; you can redistribute it
+and/or modify it under the same terms as Perl itself.
 
 =cut
+
